
API_URL = 'https://api.opinum.com:443/'
AUTHORIZATION_URL = 'https://identity.opinum.com/connect/token'
headers = {"Content-Type": "application/json",
           "Authorization": ""}


STANDARD_PUSH_DATA_URL = 'https://push.opinum.com/api/data/'

PATHS_TAGS = {'/accounts/{id}': 'Account', '/accounts': 'Account', '/accounts/{accountId}': 'Account',
              '/alerts': 'Alert', '/alerts/types': 'Alert', '/alerts/statuses': 'Alert', '/alerts/{alertId}': 'Alert',
              '/alerts/thresholds': 'Alert', '/alerts/aggregationTypes': 'Alert', '/alerts/notificationTypes': 'Alert',
              '/alerts/notificationTargets': 'Alert', '/alerts/notificationTypeTargetMapping': 'Alert',
              '/alerts/occurrences': 'AlertOccurrence', '/alerts/occurrences/statuses': 'AlertOccurrence',
              '/alerts/occurrences/status': 'AlertOccurrence', '/alerts/views': 'AlertView',
              '/alerts/views/{viewId}': 'AlertView', '/alerts/views/{id}': 'AlertView',
              '/extensions': 'CloudItemExtention', '/extensions/checkExtension': 'CloudItemExtention',
              '/dashboards': 'Dashboard', '/dashboards/{id}': 'Dashboard', '/dashboards/{id}/logo': 'Dashboard',
              '/dashboards/trendLineTypes': 'Dashboard', '/dashboards/email': 'Dashboard',
              '/dashboards/Image': 'Dashboard', '/data': 'Data', '/datapusher': 'DataPusher',
              '/dataquality/history/sources': 'DataQualityHistory',
              '/dataquality/history/variables': 'DataQualityHistory',
              '/dataquality/notifications/{sourceId}': 'DataQualityNotification',
              '/dataquality/notifications/{sourceId}/{notificationId}': 'DataQualityNotification',
              '/dataquality/overrides': 'DataQualityOverrides',
              '/dataquality/overrides/{overrideId}': 'DataQualityOverrides',
              '/dataquality/templates': 'DataQualityTemplates', '/errors/{code}': 'ErrorCode', '/errors': 'ErrorCode',
              '/events': 'Event', '/events/{id}': 'Event', '/events/eventTypes': 'Event',
              '/events/timeRangeTypes': 'Event', '/externallogin/loginproviders': 'ExternalLogin',
              '/externallogin/all/externalLogins': 'ExternalLogin',
              '/externallogin/user/externallogins': 'ExternalLogin',
              '/externallogin/account/externallogins': 'ExternalLogin',
              '/externallogin/user/externallogins/{userId}': 'ExternalLogin',
              '/externallogin/{type}/ftpCredientials': 'ExternalLogin',
              '/externallogin/{type}/basicAuth': 'ExternalLogin',
              '/externallogin/{type}/connectionString': 'ExternalLogin',
              '/externallogin/{type}/accountNameAndKey': 'ExternalLogin',
              '/externallogin/user/externallogin': 'ExternalLogin',
              '/externallogin/account/externallogin': 'ExternalLogin', '/fileTrigger/{id}': 'FileTrigger',
              '/fileTrigger/mappings': 'FileTrigger', '/fileTrigger/fileTypes': 'FileTrigger',
              '/fileTrigger/filterTypes': 'FileTrigger', '/fileTrigger/triggerTypes': 'FileTrigger',
              '/fileTrigger/generateToken': 'FileTrigger', '/fileTrigger/ingestionTypes': 'FileTrigger',
              '/fileTrigger/triggerCombinations': 'FileTrigger', '/fileTrigger': 'FileTrigger',
              '/fileTrigger/count': 'FileTrigger', '/form': 'Form', '/form/{id}': 'Form', '/form/fieldTypes': 'Form',
              '/form/entityTypes': 'Form', '/gateways': 'Gateway', '/gateways/{id}': 'Gateway',
              '/gateways/count': 'Gateway', '/groups/{groupId}': 'Group', '/groups': 'Group',
              '/groups/{groupId}/sites': 'Group', '/invitation/roles': 'Invitation', '/invitation': 'Invitation',
              '/invitation/sendBack': 'Invitation', '/invitation/isEmailUnique': 'Invitation',
              '/invitationNotification/{invitationId}': 'InvitationNotification', '/invoices': 'Invoice',
              '/invoices/types': 'Invoice', '/invoices/statuses': 'Invoice', '/invoices/{invoiceId}': 'Invoice',
              '/languages': 'Language', '/localization/{languageCode}': 'Localization',
              '/localization/label': 'Localization', '/localization': 'Localization', '/units': 'Metadata',
              '/sitetypes': 'Metadata', '/timezones': 'Metadata', '/aggregates': 'Metadata', '/unitFamily': 'Metadata',
              '/energyTypes': 'Metadata', '/granularity': 'Metadata', '/sourceTypes': 'Metadata',
              '/timePeriods': 'Metadata', '/energyUsages': 'Metadata', '/gatewayTypes': 'Metadata',
              '/quantityTypes': 'Metadata', '/variableTypes': 'Metadata',
              '/notificationMessageStatusHistory/{trackingId}': 'NotificationMessageStatusHistory',
              '/notificationMessageStatusHistory/Plivo': 'NotificationMessageStatusHistory',
              '/notificationMessageStatusHistory/Sendgrid': 'NotificationMessageStatusHistory',
              '/primaryConsumption/source/{sourceId}': 'PrimaryConsumption',
              '/primaryConsumption/{id}': 'PrimaryConsumption', '/primaryConsumption': 'PrimaryConsumption',
              '/reconstruction': 'Reconstruction', '/reconstruction/preview': 'Reconstruction', '/reports': 'Report',
              '/reports/{reportId}': 'Report', '/reports/occurrences': 'Report', '/reports/notificationTypes': 'Report',
              '/reports/notificationFormats': 'Report', '/reports/notificationTargets': 'Report',
              '/reports/notificationSendAsTypes': 'Report', '/reports/exports/{reportId}': 'Report',
              '/reports/views': 'ReportView', '/reports/views/{viewId}': 'ReportView',
              '/reports/views/{id}': 'ReportView', '/schedule/site/{siteId}': 'Schedule',
              '/schedule/{scheduleId}/device': 'Schedule', '/schedule/{scheduleId}/levels': 'Schedule',
              '/schedule/{scheduleId}/exceptions': 'Schedule', '/schedule': 'Schedule',
              '/schedule/{scheduleId}/ticks/year/{year}/week/{weekNumber}': 'Schedule',
              '/schedule/{scheduleId}': 'Schedule', '/schedule/{scheduleId}/exceptions/{exceptionId}': 'Schedule',
              '/schedule/{scheduleId}/ticks/year/{year}/week/{week}/copy/year/{targetYear}/week/{targetWeek}/{weekCount}': 'Schedule',
              '/sims/{id}': 'Sim', '/sims/providers': 'Sim', '/sims': 'Sim', '/sims/count': 'Sim', '/sites': 'Site',
              '/sites/{siteId}': 'Site', '/sites/{siteId}/users': 'Site', '/sites/{siteId}/user/{userId}': 'Site',
              '/sites/account': 'Site', '/sources': 'Source', '/sources/{sourceId}': 'Source',
              '/sources/{sourceId}/user/{userId}': 'Source', '/sources/site': 'Source', '/sources/account': 'Source',
              '/sources/{sourceId}/dataQuality/{historyId}': 'SourceDataQuality',
              '/sources/{sourceId}/dataQuality': 'SourceDataQuality', '/sources/{sourceId}/history': 'SourceHistory',
              '/sources/{sourceId}/indexes/{variableId}': 'SourceIndex',
              '/sources/{sourceId}/indexes/{variableId}/history': 'SourceIndex',
              '/sources/{sourceId}/notes': 'SourceNote', '/sources/{sourceId}/notes/{noteId}': 'SourceNote',
              '/sources/views': 'SourceView', '/sources/views/{viewId}': 'SourceView',
              '/sources/views/{id}': 'SourceView', '/storage': 'Storage', '/storage/tags': 'Storage',
              '/storage/{id}': 'Storage', '/storage/totalUsage': 'Storage', '/storage/file': 'Storage',
              '/storage/exportTemplates': 'Storage', '/storage/canupload/{fileSize}': 'Storage', '/tags': 'Tag',
              '/user/info': 'User', '/user/roles': 'User', '/user/features': 'User', '/user/info/{id}': 'User',
              '/user/role/{id}': 'User', '/user/adminUsers': 'User', '/user/claim/{id}': 'User',
              '/user/rolemapping': 'User', '/user/usermapping': 'User', '/user/getUnitsPreference': 'User',
              '/user/canAccessToSite/{siteId}': 'User', '/user/canAccessToGroup/{groupId}': 'User',
              '/user/canAccessToSource/{sourceId}': 'User', '/user/lock': 'User', '/user/save': 'User',
              '/user/create': 'User', '/user/delete': 'User', '/user/remove': 'User', '/user/password': 'User',
              '/user/saveUser': 'User', '/user/claims/add': 'User', '/user/roles/save': 'User',
              '/user/claims/edit': 'User', '/user/claims/remove': 'User', '/user/isEmailUnique': 'User',
              '/user/saveUnitsPreference': 'User', '/user/resetPassword/{userId}': 'User', '/user/users': 'User',
              '/account/users': 'UserAccount', '/account/features': 'UserAccount',
              '/account/usermapping': 'UserAccount', '/account/secondaryAccesses': 'UserAccount',
              '/account/usermapping/users': 'UserAccount', '/account/user/claim/{userId}': 'UserAccount',
              '/account/usersForAccount/{id}': 'UserAccount', '/account/user/{userId}/features': 'UserAccount',
              '/account': 'UserAccount', '/account/user/{userId}/secondaryAccesses': 'UserAccount',
              '/account/user/{userId}': 'UserAccount', '/account/user/claims/edit': 'UserAccount',
              '/account/user/claims/add': 'UserAccount', '/account/user/{userId}/claims/add': 'UserAccount',
              '/account/user/{userId}/claims/remove': 'UserAccount', '/account/user/create': 'UserAccount',
              '/account/user/{userId}/lock': 'UserAccount', '/account/user/{userId}/resetpassword': 'UserAccount',
              '/account/user/claims/remove': 'UserAccount', '/variables': 'Variable',
              '/variables/periodTypes': 'Variable', '/variables/types': 'Variable',
              '/variables/source/{sourceId}': 'Variable', '/sources/{sourceId}/variables/{variableId}': 'Variable',
              '/variables/types/energytype/{energyTypeId}': 'Variable',
              '/variables/types/{variableTypeId}/energytype/{energyTypeId}': 'Variable',
              '/sources/{sourceId}/variables/{variableId}/calculated/evaluate': 'Variable',
              '/sources/{sourceId}/variables/{variableId}/calculated/validate': 'Variable',
              '/sources/{sourceId}/variables/{variableId}/calculated/sampleData': 'Variable',
              '/account/webhooks/test/{id}': 'Webhook', '/account/webhooks/{id}/history': 'Webhook',
              '/account/webhooks': 'Webhook', '/account/webhooks/{id}': 'Webhook', '/account/webhooks/test': 'Webhook'}



import opisense_client as oc
import datetime
import random
import json
from time import sleep

""" Inputs """

# set test inputs
opisense_site = {
    "clientData": {},
    "typeId": 15,
    "street": "Street test site",
    "postalCode": "0000",
    "city": "Test City",
    "country": "Test Country",
    "timeZoneId": "UTC",
    "name": "Test Site"
}

opisense_source = {
    "siteId": 0,
    "serialNumber": "Source_test_SN",
    "clientData": {},
    "energyTypeId": 1,
    "name": "Source_test",
    "sourceTypeId": 72,
    "timeZoneId": "UTC"
}

opisense_variable = {
    "name": "Variable Test",
    "sourceId": 0,
    "isDefault": True,
    "unitId": 8,
    "granularity": 1,
    "granularityTimeBase": "Second",
    "quantityType": "Instantaneous",
    "mappingConfig": "mapping_config"
}

# get Opisense credentials from file
credentials_file_path = 'opisense_client/tests/test_credentails.JSON'

with open(credentials_file_path, encoding='utf-8') as file:
    credentials = json.load(file)
user_credentials = credentials['user']
api_credentials = credentials['api']

""" TESTS """


def test_OpisenseObject():
    site = oc.OpisenseObject(type='site', opisense_object=opisense_site)
    assert site.type == 'site'
    assert site.id == None
    assert site.content == {'clientData': {},
                            'typeId': 15,
                            'street': 'Street test site',
                            'postalCode': '0000',
                            'city': 'Test City',
                            'country': 'Test Country',
                            'timeZoneId': 'UTC',
                            'name': 'Test Site'}

    assert site.json() == '{"clientData": {}, "typeId": 15, "street": "Street test site", "postalCode": "0000", "city": "Test City", "country": "Test Country", "timeZoneId": "UTC", "name": "Test Site"}'

    assert site.api_path == 'sites'


def test_ApiFilter():
    filter = oc.ApiFilter('data', granularity='Hour',
                          date_from='2019-01-01T23:00:00',
                          date_to='2019-01-02T23:00:00')

    assert filter.filters['from'] == '2019-01-01T23:00:00'
    assert filter.filters['to'] == '2019-01-02T23:00:00'
    assert filter.filters == {'granularity': 'Hour', 'from': '2019-01-01T23:00:00', 'to': '2019-01-02T23:00:00'}
    assert filter.path == 'data'


def test_authorize():
    opisense_token = oc.authorize(user_credentials, api_credentials, feedback=True)
    assert type(opisense_token) == str


# If Authorize is tested and OK, get one uniaue token for the remaining tests
opisense_token = oc.authorize(user_credentials, api_credentials, feedback=True)


def test_GET():
    assert oc.GET(opisense_token, oc.ApiFilter('units')).status_code == 200


def test_OpisenseObject_Calls():
    # POST
    site = oc.OpisenseObject(type='site', opisense_object=opisense_site)
    result = site.POST(opisense_token)
    assert result.status_code == 200
    site.id = result.json()
    assert type(result.json()) == int

    source = oc.OpisenseObject(type='source', opisense_object=opisense_source)
    source.content['siteId'] = site.id
    result = source.POST(opisense_token)
    source.id = result.json()['id']
    assert result.status_code == 200
    assert type(result.json()) == dict

    variable = oc.OpisenseObject(type='variable', opisense_object=opisense_variable)
    variable.content['sourceId'] = source.id
    result = variable.POST(opisense_token, parent_id=source.id)
    variable.id = result.json()['id']
    assert result.status_code == 200
    assert type(result.json()) == dict

    # PUT
    source.content['name'] = 'Source New Name'
    result = source.PUT(opisense_token)
    assert result.status_code == 204
    assert oc.GET(opisense_token, oc.ApiFilter('sources', id=source.id), json_output=True)[0][
               'name'] == 'Source New Name'

    variable.content['name'] = 'Variable New Name'
    result = variable.PUT(opisense_token, parent_id=source.id)
    assert result.status_code == 204
    assert oc.GET(opisense_token, oc.ApiFilter('variables', id=variable.id), json_output=True)[0][
               'name'] == 'Variable New Name'

    # DELETE
    result = variable.DELETE(opisense_token,
                             force_path='/sources/{}/variables/'.format(variable.content["sourceId"]))
    assert result.status_code == 204

    result = source.DELETE(opisense_token)
    assert result.status_code == 204

    result = site.DELETE(opisense_token)
    assert result.status_code == 204


def test_StandardData_POST():
    variableId = 991929

    now = datetime.datetime.now()
    start = now
    datapoints = oc.DataPoints(now, random.randint(5, 25))
    for _ in range(5):
        now += datetime.timedelta(minutes=5)
        datapoints.__add__(now, random.randint(5, 25))

    data = oc.StandardData(datapoints,
                           variableId=variableId)

    result = data.POST(opisense_token)
    assert result.status_code == 204

    sleep(5)

    result = oc.GET(opisense_token, oc.ApiFilter('data', variableId=991929,
                                                 date_from=start.strftime('%Y-%m-%dT%H:%M:%S%z')))
    assert result.status_code == 200
    assert result.json().__len__() == 6



from .inputs import *
from .objects import *
from .http import *


def change_urls(api_url=None, authorization_url=None, standard_push_data_url=None):
    """
    Change API URLS

    :param api_url: new API URL
    :param authorization_url: new AUTHORIZATION URL
    :param standard_push_data_url: new STANDARD PUSH DATA URL
    """
    import opisense_client as oc
    # We need to change all those because these variables are string and are consequently copied and not referenced
    if api_url is not None:
        oc.API_URL = api_url
        oc.inputs.API_URL = api_url
        oc.http.API_URL = api_url
    if authorization_url is not None:
        oc.AUTHORIZATION_URL = authorization_url
        oc.inputs.AUTHORIZATION_URL = authorization_url
        oc.http.AUTHORIZATION_URL = authorization_url
    if standard_push_data_url is not None:
        oc.STANDARD_PUSH_DATA_URL = standard_push_data_url
        oc.inputs.STANDARD_PUSH_DATA_URL = standard_push_data_url
        oc.objects.STANDARD_PUSH_DATA_URL = standard_push_data_url